@@ -305,6 +305,51 @@ class PinholeCameraModel(object):
         """
         return self.P[1, 1]
 
+    @property
+    def Tx(self):
+        """Return Tx.
+
+        For monocular cameras, Tx = Ty = Tz = 0.
+        For a stereo pair, the fourth column [Tx Ty Tz]' is related to the
+        position of the optical center of the second camera in the first
+        camera's frame.
+
+        Returns
+        -------
+        Tx : numpy.float32
+        """
+        return self.P[0, 3]
+
+    @property
+    def Ty(self):
+        """Return Ty.
+
+        For monocular cameras, Tx = Ty = Tz = 0.
+        For a stereo pair, the fourth column [Tx Ty Tz]' is related to the
+        position of the optical center of the second camera in the first
+        camera's frame.
+
+        Returns
+        -------
+        Ty : numpy.float32
+        """
+        return self.P[1, 3]
+
+    @property
+    def Tz(self):
+        """Return Tz.
+
+        For monocular cameras, Tx = Ty = Tz = 0.
+        For a stereo pair, the fourth column [Tx Ty Tz]' is related to the
+        position of the optical center of the second camera in the first
+        camera's frame.
+
+        Returns
+        -------
+        Tz : numpy.float32
+        """
+        return self.P[2, 3]
+
     @property
     def fov(self):
         """Property of fov.
@@ -1355,6 +1400,52 @@ class PinholeCameraModel(object):
                 return uv, np.where(valid_indices)[0]
         return uv
 
+    def points_to_depth(self, points, depth_value=0.0):
+        """Return depth image from 3D points.
+
+        Parameters
+        ----------
+        points : numpy.ndarray
+            batch of xyz point (batch_size, 3) or (height, width, 3).
+        depth_value : float
+            default depth value.
+
+        Returns
+        -------
+        depth : numpy.ndarray
+            projected depth image.
+        """
+        if points.shape == (self.height, self.width, 3):
+            points = points.reshape(-1, 3)
+        uv, indices = self.batch_project3d_to_pixel(
+            points,
+            project_valid_depth_only=True,
+            return_indices=True)
+        # round off
+        uv = np.array(uv + 0.5, dtype=np.int32)
+        depth = depth_value * np.ones((self.height, self.width), 'f')
+        depth.reshape(-1)[self.flatten_uv(uv)] = points[indices][:, 2]
+        return depth
+
+    def depth_to_points(self, depth):
+        """Convert depth image to point clouds.
+
+        Parameters
+        ----------
+        depth : numpy.ndarray
+            depth image.
+
+        Returns
+        -------
+        points : numpy.ndarray
+            return shape is (width, height, 3).
+        """
+        uv = self.flattened_pixel_locations_to_uv(
+            np.arange(self.width * self.height))
+        points = self.batch_project_pixel_to_3d_ray(
+            uv, depth=depth)
+        return points.reshape(self.height, self.width, 3)
+
     def get_view_frustum(self, max_depth=1.0,
                          translation=np.zeros(3),
                          rotation=np.eye(3)):
@@ -1487,23 +1578,48 @@ class PinholeCameraModel(object):
             (flat_pixel_locations % self.width).reshape(-1, 1),
             (flat_pixel_locations.T // self.width).reshape(-1, 1)])
 
-    def dump(self, output_filepath):
+    def dump(self, output_filepath, save_original=True):
         """Dump this camera's parameter to yaml file.
 
         Parameters
         ----------
         output_filepath : str or pathlib.Path
             output path
+        save_original : bool
+            If `False`, save resized camera info.
         """
+        if save_original is True:
+            width = self._full_width
+            height = self._full_height
+            K = self.full_K
+            P = self.full_P
+            binning_x = self._binning_x
+            binning_y = self._binning_y
+            x_offset = self.roi[1]
+            y_offset = self.roi[0]
+            roi_height = self.roi[2] - self.roi[0]
+            roi_width = self.roi[3] - self.roi[1]
+        else:
+            width = int(self._width / self._binning_x)
+            height = int(self._height / self._binning_y)
+            K = self.K
+            P = self.P
+            binning_x = 1
+            binning_y = 1
+            x_offset = 0
+            y_offset = 0
+            roi_height = 0
+            roi_width = 0
+
         camera_data = "\n".join([
-                "image_width: %d" % self._full_width,
-                "image_height: %d" % self._full_height,
+                "image_width: %d" % width,
+                "image_height: %d" % height,
                 "camera_name: " + self.name,
                 "camera_matrix:",
                 "  rows: 3",
                 "  cols: 3",
                 "  data: " + format_mat(
-                    np.array(self.full_K.reshape(-1), dtype=np.float64), 5),
+                    np.array(K.reshape(-1), dtype=np.float64), 5),
                 "distortion_model: " + self.distortion_model,
                 "distortion_coefficients:",
                 "  rows: 1",
@@ -1520,14 +1636,14 @@ class PinholeCameraModel(object):
                 "  rows: 3",
                 "  cols: 4",
                 "  data: " + format_mat(
-                    np.array(self.full_P.reshape(-1), dtype=np.float64), 5),
-                "binning_x: %f" % self._binning_x,
-                "binning_y: %f" % self._binning_y,
+                    np.array(P.reshape(-1), dtype=np.float64), 5),
+                "binning_x: %f" % binning_x,
+                "binning_y: %f" % binning_y,
                 "roi:",
-                "  x_offset: %d" % self.roi[1],
-                "  y_offset: %d" % self.roi[0],
-                "  height: %d" % (self.roi[2] - self.roi[0]),
-                "  width: %d" % (self.roi[3] - self.roi[1]),
+                "  x_offset: %d" % x_offset,
+                "  y_offset: %d" % y_offset,
+                "  height: %d" % roi_height,
+                "  width: %d" % roi_width,
                 ""
             ])
         with open(str(output_filepath), 'w') as f:
